//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#ifndef GLES20STD_FS
#define GLES20STD_FS

#ifdef FRAGMENT_SHADER

vec3 unpackNormal(sampler2D texture, vec2 coords) {
    vec3 N = normalize(varNormal);
    vec3 T = normalize(varTangent);
    vec3 B = normalize(cross(varNormal, varTangent) * varBitangentSign);
    mat3 TBN = mat3(T, B, N);
    vec3 nm = texture2D(texture, coords).xyz * 2.0 - vec3(1.0);
    return normalize(TBN * normalize(nm));
}

#if defined (sc_UseFramebufferFetchMarker)
uniform float __sc_framebufferFetchMarker;
#endif

vec4 getFramebufferColor() {
#ifdef sc_FramebufferFetch

    #if defined(GL_EXT_shader_framebuffer_fetch)
        vec4 result = gl_LastFragData[0].rgba;
    #elif defined(GL_ARM_shader_framebuffer_fetch)
        vec4 result = gl_LastFragColorARM;
    #else
        #error sc_FramebufferFetch but no ext found!
    #endif

    #if defined (sc_UseFramebufferFetchMarker)
    // HACK: this is the only way we can detect currently whether a shader uses framebuffer fetch.
    return result + vec4(0.000001, 0.0, 0.0, 0.0) * __sc_framebufferFetchMarker;
    #else
    return result;
    #endif

#else

    return texture2D(sc_ScreenTexture, varScreenPos.xy / varScreenPos.w * 0.5 + 0.5).rgba;

#endif
}

#define CLAMP_UV(uv, uvMin, uvMax, useClampToBorder, borderClampFactor) {\
    float clampedUv = clamp(uv, uvMin, uvMax);\
\
    if (useClampToBorder) {\
        borderClampFactor *= step(abs(uv - clampedUv), 0.00001);\
    }\
\
    uv = clampedUv;\
}

#define SOFTWARE_WRAP(uv, softwareWrapMode, useClampToBorder, clampToBorderFactor) {\
    if (softwareWrapMode == SOFTWARE_WRAP_MODE_CLAMP_TO_EDGE || softwareWrapMode == SOFTWARE_WRAP_MODE_CLAMP_TO_BORDER) {\
        CLAMP_UV(uv, 0.0, 1.0, useClampToBorder, clampToBorderFactor); \
    } else if (softwareWrapMode == SOFTWARE_WRAP_MODE_REPEAT) {\
        uv = fract(uv); \
    } else if (softwareWrapMode == SOFTWARE_WRAP_MODE_MIRRORED_REPEAT) {\
        float uvFract = fract(uv);\
        float uvInt = uv - uvFract;\
        float uvOdd = step(0.25, fract(uvInt * 0.5));\
        uv = mix(uvFract, 1.0 - uvFract, saturate(uvOdd));\
    }    \
}

#define SAMPLE_TEXTURE(tex, sampler, uv, useUvTransform, uvTransform, softwareWrapModeU, softwareWrapModeV, useUvMinMax, uvMinMax, useClampToBorder, borderColor, mipBias) {\
    float clampToBorderFactor = 1.0;\
\
    SOFTWARE_WRAP(uv.x, softwareWrapModeU, useClampToBorder && !useUvMinMax, clampToBorderFactor);\
    SOFTWARE_WRAP(uv.y, softwareWrapModeV, useClampToBorder && !useUvMinMax, clampToBorderFactor);\
\
    if (useUvMinMax) {\
        CLAMP_UV(uv.x, uvMinMax.x, uvMinMax.z, useClampToBorder, clampToBorderFactor);\
        CLAMP_UV(uv.y, uvMinMax.y, uvMinMax.w, useClampToBorder, clampToBorderFactor);\
    }\
\
    if (useUvTransform) {\
        uv = vec2(uvTransform * vec3(uv, 1.0));\
    }\
\
    tex = texture2D(sampler, uv, mipBias);\
\
    if (useClampToBorder) {\
        tex = mix(borderColor, tex, clampToBorderFactor);\
    }\
}

#endif //FRAGMENT_SHADER


#endif //GLES20STD_FS
