//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#ifndef GLES20STD_SHADOWS
#define GLES20STD_SHADOWS

#include <std.glsl>

#if sc_ExporterVersion < 74   // HACK 05/17/2018 for old Vivante shaders. Search for similar comments in code and std shaders. Remove this block once all old assets are dead.
#ifndef SC_RENDERING_SPACE_OBJECT
#define SC_RENDERING_SPACE_OBJECT    1
#define SC_RENDERING_SPACE_WORLD     2
#define SC_RENDERING_SPACE_SCREEN    3
#define SC_RENDERING_SPACE_SCREEN_MV 4
#endif
#endif

#ifdef sc_ProjectiveShadowsCaster
uniform float sc_ShadowDensity;

#ifdef FRAGMENT_SHADER
float getShadowAlpha(vec4 color, float alphaTestThreshold) {
    float shadowAlpha = 1.0;
#if defined(sc_BlendMode_Normal) || defined(sc_BlendMode_AlphaToCoverage)
    shadowAlpha = color.a;
#elif defined(sc_BlendMode_PremultipliedAlpha)
    shadowAlpha = clamp(color.a * 2.0, 0.0, 1.0);
#elif defined(sc_BlendMode_AddWithAlphaFactor)
    shadowAlpha = clamp(dot(color.rgb, vec3(color.a)), 0.0, 1.0);
#elif defined(sc_BlendMode_AlphaTest)
    shadowAlpha = color.a < alphaTestThreshold ? 0.0 : 1.0;
#elif defined(sc_BlendMode_Multiply)
    shadowAlpha = (1.0 - dot(color.rgb, vec3(0.33333))) * color.a;
#elif defined(sc_BlendMode_MultiplyOriginal)
    shadowAlpha = (1.0 - clamp(dot(color.rgb, vec3(1.0)), 0.0, 1.0)) * color.a;
#elif defined(sc_BlendMode_ColoredGlass)
    shadowAlpha = (1.0 - dot(color.rgb, vec3(0.33333))) * color.a;
#elif defined(sc_BlendMode_Add)
    shadowAlpha = clamp(dot(color.rgb, vec3(1.0)), 0.0, 1.0);
#elif defined(sc_BlendMode_AddWithAlphaFactor)
    shadowAlpha = clamp(dot(color.rgb, vec3(1.0)), 0.0, 1.0) * color.a;
#elif defined(sc_BlendMode_Screen)
    shadowAlpha = dot(color.rgb, vec3(0.33333)) * color.a;
#elif defined(sc_BlendMode_Min)
    shadowAlpha = (1.0 - clamp(dot(color.rgb, vec3(1.0)), 0.0, 1.0));// * color.a;
#elif defined(sc_BlendMode_Max)
    shadowAlpha = clamp(dot(color.rgb, vec3(1.0)), 0.0, 1.0);// * color.a;
#endif // blend modes
    return shadowAlpha;
}

vec4 getShadowColor(float alpha) {
    float shadowTerm = sc_ShadowDensity * alpha;
#if (sc_ExporterVersion < 68)
    shadowTerm = 1.0 - shadowTerm;
#endif // sc_ExporterVersion
    return vec4(shadowTerm, shadowTerm, shadowTerm, 1.0);
}
#endif // FRAGMENT_SHADER
#endif // sc_ProjectiveShadowsCaster

#ifdef sc_ProjectiveShadowsReceiver
varying vec2 varShadowTex;

uniform sampler2D sc_ShadowTexture;
uniform mat4 sc_ProjectorMatrix;

#if (sc_ExporterVersion >= 68)
uniform float sc_ShadowDensity;
#endif // sc_ExporterVersion

#ifdef VERTEX_SHADER
vec2 getProjectedTexCoords(vec4 vertexPosition) {
#if (sc_RenderingSpace == SC_RENDERING_SPACE_WORLD)
    vec4 worldPos = vertexPosition;
#elif (sc_RenderingSpace == SC_RENDERING_SPACE_OBJECT)
    vec4 worldPos = sc_ModelMatrix * vertexPosition;
#else
#error Unsupported sc_RenderingSpace in getProjectedTexCoords()!
#endif // sc_RenderingSpace
    vec4 projectedUVs = sc_ProjectorMatrix * worldPos;
    return projectedUVs.xy / projectedUVs.w * 0.5 + 0.5;
}
#endif // VERTEX_SHADER

#ifdef FRAGMENT_SHADER
float getShadowSample() {
    float result = 1.0;
    if (varShadowTex.x > 0.0 && varShadowTex.x < 1.0 && varShadowTex.y > 0.0 && varShadowTex.y < 1.0) {
        result = texture2D(sc_ShadowTexture, varShadowTex).r;
#if (sc_ExporterVersion >= 68)
        result = 1.0 - result * sc_ShadowDensity;
#endif // sc_ExporterVersion
    }
    return result;
}
#endif // FRAGMENT_SHADER
#endif // sc_ProjectiveShadowsReceiver

#if sc_ExporterVersion < 74   // HACK 05/17/2018 for old Vivante shaders. Search for similar comments in code and std shaders. Remove this block once all old assets are dead.
#undef SC_RENDERING_SPACE_OBJECT   
#undef SC_RENDERING_SPACE_WORLD    
#undef SC_RENDERING_SPACE_SCREEN   
#undef SC_RENDERING_SPACE_SCREEN_MV
#endif

#endif //GLES20STD_SHADOWS
